%Copyright (C) 2005  Lastenet-Ramel
%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

function newmap=brighten(map,beta)
%  
% newmap=brighten(map,beta)
%
% Brighten or darken the colormap.
%
% brighten increases or decreases the color intensities in a colormap.
% The modified colormap is brighter if 0 < beta < 1 and darker if -1 < beta < 0.
% 
% - brighten(beta) replaces the current colormap with a brighter or darker colormap
% of essentially the same colors. brighten(beta), followed by brighten(-beta),
% where beta < 1, restores the original map.
% 
% - newmap = brighten(beta) returns a brighter or darker version of the current colormap without changing the display.
% 
% - newmap = brighten(map,beta) returns a brighter or darker version of the colormap map without changing the display. 
% 
% The values in the colormap are raised to the power of gamma, where gamma is:
% 1-beta when beta>0
% 1/(1+beta) when <=0


if (nargin>2 | nargin<1)
	disp('too many or few arguments');
	break;
end

if (nargin == 1)
    beta=map;
    map=colormap;
end

if (beta <= 0)
    gamma=1/(1+beta);
else
    gamma=1 - beta;
end

map=map.^gamma;

if nargout == 0
   colormap(map)
else
   newmap = map;
end