%Copyright (C) 2005  Berge-Gladel

%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

function image2 = bwperim(image1,n)

%   BWPERIM Determine perimeter of objects in binary image.

%   The image that you give in input must only contains 0 and 1
%   (image1) => black and white

%   image2 = bwperim(image1,n) returns a binary image. 
%   This image containing only the perimeter pixels of objects 
%   in the input image BW1.

%   A pixel is part of the perimeter if its value is 1 and there
%   is at least one zero-valued pixel in its neighborhood. 
%   0 => black
%   1 => white

%   n can have a value of either 4 or 8, where 4 specifies 4-connected
%   neighborhoods and 8 specifies 8-connected neighborhoods; if
%   the argument if omitted, it defaults to 4.
%
%   Class Support
%   -------------
%   The input image BW1 can be of class double or uint8.  The
%   output image BW2 is of class uint8.
%
%   Example
%   -------
%       [image1,map] = imread('exemple.tif');
%       image2 = bwperim(image1,8);
%       imshow(image1)
%       imshow(image2)                


s = size(image1);
nl = s(1);
nc = s(2);

if nargin<2
    n = 4; 
end

if n~=8 & n~=4
    error('n ne peut que avoir comme valeur 4 ou 8');
end

if n==4
   for i=2:(nl-1)
       for j=2:(nc-1)
           if image1(i,j)==0
               image2(i,j)=0;
           else
               image2(i,j)=1;
           end
           if ((image1(i,j)==image1(i-1,j)) & (image1(i,j)==image1(i,j+1)) & (image1(i,j)==image1(i+1,j)) & (image1(i,j)==image1(i,j-1)))
               image2(i,j)=0;
           end
       end
   end
           
    
else % ici n est egal a 8
    for i=2:(nl-1)
       for j=2:(nc-1)
           if image1(i,j)==0
               image2(i,j)=0;
           else
               image2(i,j)=1;
           end
           if (image1(i,j)==image1(i-1,j)) & (image1(i,j)==image1(i-1,j+1)) & (image1(i,j)==image1(i,j+1)) & (image1(i,j)==image1(i+1,j+1))& (image1(i,j)==image1(i+1,j))& (image1(i,j)==image1(i+1,j-1))& (image1(i,j)==image1(i,j-1))& (image1(i,j)==image1(i-1,j-1))
               image2(i,j)=0;
           end
       end
   end    
end
