%Copyright (C) 2005  Lastenet-Ramel
%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

function IM2 = imclearborder(IM,CONN)
%
% IM2 = imclearborder(IM,CONN)
%
% Suppress light structures connected to image border
% 
% - IM2 = imclearborder(IM) suppresses structures that are lighter than their
% surroundings and that are connected to the image border. IM can be an intensity
% or binary image. The output image, IM2, is intensity or binary, respectively.
% The default connectivity is 8 for two dimensions, 26 for three dimensions.
%
% - IM2 = imclearborder(IM,CONN) specifies the desired connectivity. CONN can have
% any of the following scalar values. 

if (nargin>2 | nargin<1)
	disp('too many or few arguments');
	break;
end

if (nargin==1)

	if(length(size(IM))==2)
		CONN=8;
	elseif(length(size(IM))==3)
		CONN=26;
	else
		disp('too many dimensions');
		break;
	end 

end

[N,M]=size(IM);

mask=zeros(N,M);
mask(1,:)=IM(1,:);
mask(end,:)=IM(end,:);
mask(:,1)=IM(:,1);
mask(:,end)=IM(:,end);


IMreconstruct = imreconstruct(mask,IM,CONN);

IM2=IM - IMreconstruct;