%Copyright (C) 2005  Caius-Lavalley

%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

% IMCOMPLEMENT Complement image.
%     IM2 = IMCOMPLEMENT(IM) computes the complement of the image IM.  IM
%     can be a binary, intensity, or RGB image.  IM2 has the same class and
%     size as IM.
%  
%     In the complement of a binary image, zeros become ones and ones
%     become zeros; black and white are reversed.  In the complement of an
%     intensity or RGB image, each pixel value is subtracted from the
%     maximum pixel value supported by the class (or 1.0 for
%     double-precision images), and the difference is used as the pixel
%     value in the output image.  In the output image, dark areas become
%     lighter and light areas become darker.
%  
%     Note
%     ----
%     If IM is a double intensity or RGB image, you can use the expression
%     1-IM instead of this function.  If IM is a binary image, you can use
%     the expression ~IM instead of this function.
%
%   This function cannot be used now without a second parameter (limitations
%   of type management in Octave)
%
%   Usable version :
%     IM2 = IMCOMPLEMENT(IM,format)
%       format :
%           'RGBnorm'   normalized RGB [0.0-1.0]
%           'RGBint'    RGB [0.0-1.0]
%           'BW'        Binary image [0-1]
%           'intensity' Intensity image [0-255]
function [imgRes] = imcomplementdemo (img, format)


% 1 for Matlab, 0 for Octave
matLab = 1 ;


% Check the number of arguments
if nargin ~= 1 & nargin ~= 2
	error ('Invalid number of arguments : imcomplementdemo <image> [<format>]')
end

% Check image format if not given by user
if nargin == 1
    % RGB : check if there are non-integer values
    % (method used by Matlab)
    if isrgb(img)
        %% Matlab only
        if matLab
            imgDb = double(img) ;
        else
            imgDb = img
        end
        if max(max(max(floor(imgDb) ~= img))) == 1
            format = 'RGBnorm' ;
        else
            format = 'RGBint' ;
        end
    % (test before isgray : isgray always returns true if isbw returns true)
    elseif isbw(img)
        format = 'BW' ;
    elseif isgray(img)
        format = 'intensity' ;
    else
        error ('Unknown image format') ;
    end
end

% Check the number of dimensions
nbDim = length(size(img)) ;
if (strcmp(format,'RGBint') | strcmp(format,'RGBnorm')) & nbDim ~= 3
    error ('Incorrect number of dimensions for a RGB image : sould be 3') ;
end
if (strcmp(format,'intensity') | strcmp(format,'BW')) & nbDim ~= 2
    error ('Incorrect number of dimensions for a binary or intensity image : sould be 2') ;
end


% Depending on the format
if strcmp (format,'RGBnorm')
    imgRes = 1 - img ;
elseif strcmp (format,'RGBint')
    %% Matlab only
    if matLab
        img = double(img) ;
    end
    imgRes(:,:,1) = 255 - img(:,:,1) ;
    imgRes(:,:,2) = 255 - img(:,:,2) ;
    imgRes(:,:,3) = 255 - img(:,:,3) ;
    %% Matlab only
    if matLab
        imgRes = uint8(imgRes) ;
    end
elseif strcmp (format,'intensity')
    %% Matlab only
    if matLab
        img = double(img) ;
    end
    imgRes = 255 - img ;
    %% Matlab only
    if matLab
        imgRes = uint8(imgRes) ;
    end
elseif strcmp (format,'BW')
    %% Matlab only
    if matLab
        img = double(img) ;
    end
    imgRes = 1 - img ;
    %% Matlab only
    if matLab
        imgRes = uint8(imgRes) ;
    end
else
    error ('Internal error : incorrect format') ;
end

%% Octave
end