%Copyright (C) 2005  Grucker-Hardy

%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

%#    IMERODE Erodes image.
%#    IM2 = IMERODE(IM,SE) erodes the grayscale, binary, or packed binary 
%#    image IM, returning the eroded image, IM2.  SE is an array of 0s
%#    and 1s that specifies the structuring element.
 
%#    If IM is logical and the structuring element is flat, IMERODE
%#    performs binary erosion; otherwise it performs grayscale erosion.

function [IMG_R,IMG_G,IMG_B] = imerode(varargin)

if(nargin ~= 2 && nargin ~= 4)
	usage("IMG = imdilate(im, mask) OU [R,G,B] = imdilate(r,g,b, mask)");
	
elseif(nargin == 2)
    	im = varargin{1};
    	if ~isnumeric(im) | ~isreal(im)
        	error('im doit etre une matrice de reels.');
    	end
    	mask = varargin{2};
    	if ~isnumeric(mask) | ~isreal(mask)
        	error('mask doit etre une matrice de reels.');
    	end
	
	[h,l]= size(mask);
	[n,m]= size(im);
	im = im + ones(n,m);

	for i=1:h
		for j=1:l
			if (mask(i,j)==0)
				mask(i,j) = 255;
			end
		end
	end
	
	
	IMG = [];

	for i=1:n	
    		for j=1:m
			if (((i-floor(h/2))>0) && ((i+floor(h/2))<(n+1)) && ((j-floor(l/2))>0) && ((j+floor(l/2))<(m+1)))
				IMG(i,j) = min(min( ( mask .* im( (max((i-floor(h/2)),1)):(min((i+floor(h/2)),n)), (max((j-floor(l/2)),1)):(min((j+floor(l/2)),m)) ))));
			else 
				IMG(i,j) = 1;
			end
    		end
	end

	IMG = IMG - ones(n,m);
	
	IMG_R = IMG;
	IMG_G = IMG;
	IMG_B = IMG;
		
else
    	imr = varargin{1};
    	if ~isnumeric(imr) | ~isreal(imr)
        	error('imr doit etre une matrice de reels.');
    	end
    	img = varargin{2};
    	if ~isnumeric(img) | ~isreal(img)
        	error('img doit etre une matrice de reels.');
    	end
    	imb = varargin{3};
    	if ~isnumeric(imb) | ~isreal(imb)
        	error('imb doit etre une matrice de reels.');
    	end
    	mask = varargin{4};
    	if ~isnumeric(mask) | ~isreal(mask)
        	error('mask doit etre une matrice de reels.');
    	end
	
	IMG_R = [];
	IMG_G = [];
	IMG_B = [];
	
	[h,l]= size(mask);

	[n,m]= size(imr);
	imr = imr + ones(n,m);
	img = img + ones(n,m);
	imb = imb + ones(n,m);
	
	for i=1:h
		for j=1:l
			if (mask(i,j)==0)
				mask(i,j) = 255;
			end
		end
	end

	for i=1:n	
    	for j=1:m
		if (((i-floor(h/2))>0) && ((i+floor(h/2))<(n+1)) && ((j-floor(l/2))>0) && ((j+floor(l/2))<(m+1)))
			IMG_R(i,j) = min(min( ( mask .* imr( (max((i-floor(h/2)),1)):(min((i+floor(h/2)),n)), (max((j-floor(l/2)),1)):(min((j+floor(l/2)),m)) ))));
			IMG_G(i,j) = min(min( ( mask .* img( (max((i-floor(h/2)),1)):(min((i+floor(h/2)),n)), (max((j-floor(l/2)),1)):(min((j+floor(l/2)),m)) ))));
			IMG_B(i,j) = min(min( ( mask .* imb( (max((i-floor(h/2)),1)):(min((i+floor(h/2)),n)), (max((j-floor(l/2)),1)):(min((j+floor(l/2)),m)) ))));
		else 
			IMG_R(i,j) = 1;
			IMG_G(i,j) = 1;
			IMG_B(i,j) = 1;
		end
    	end
	end

	IMG_R = IMG_R - ones(n,m);
	IMG_G = IMG_G - ones(n,m);
	IMG_B = IMG_B - ones(n,m);
	
	
end

 

endfunction