%Copyright (C) 2005  Estrade-Mercadier

%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

function R = imfill(varargin)

## IMFILL
## 
## This function fills regions wich are pointed by arguments, and
## every holes when the mode is opted.
## 
## R = imfill (I,points,conn)
## R = imfill (I,points)
## R = imfill (I,'holes')
## 
## I input binary image
## R output binary image
## points array of pixels
## 'holes' makes fill all holes
## 
## Functions necessary to use CONNDEF
## ----------------------------------
##   size.m
##   find.m
##
## 


## -*- texinfo -*-
##
## @deftypefn {Function File} {@var{R} = } imfill (@var{I}, @var{points},@var{conn})
## @deftypefnx {Function File} {@var{R} = } imfill (@var{I}, @var{points})
## @deftypefnx {Function File} {@var{R} = } imfill (@var{I}, @var{mode})
##
## Input parameters :
##
##      @var{I}   a binary image matrix
## 
##      @var{points} vector of points which will be filled
## 
##      @var{conn} the degree of connectivity
## 
##      @var{mode}
##      @itemize @w
##       @item "holes" this option permit the program to fill all the holes
##      @end itemize
## 
##
## @end deftypefn



if nargin >1 & nargin<4
    
    I = varargin{1};
    t = size(I);
    
    s = size(varargin{2});
    
    % if the user use the holes mode
    if s(2) == 5
        
    	% in order to fill every holes we have opted this method:
	% - filling the image since the edges
	% - reverse the result
	% - make the OR of the original image and the reverse
    
        A = zeros(t);
        A(1:t(1),1)=1;
        A(1:t(1),t(2))=1;
        A(1,1:t(2))=1;
        A(t(1),1:t(2))=1;
        
        [x y] = find(A==1);
        
        A = ~(imfill(I,[x y],4));
        
        R = A | I;
        
    % if the user gives some points to fill
    else
        
        points = varargin{2};
        s = size(points);
	% initialization of connectivity degree
        conn = 4;
        
	% if the user gives the connectivity
        if nargin > 2
            conn = varargin{3};
        end
        
	% extraction of the first point
        x = points(1,1);
        y = points(1,2);
        
	% test of the presence in the image
        if (x>0) & (x<=t(1)) & (y>0) & (y<=t(2))
                
                if (I(x,y) == 0)
                    
                    I(x,y) = 1;
                    
		    % creation of new points in order to filling neighbors
		    
		    % copy of remaining points
                    if s(1)>1
                        P(conn+1:conn-1+s(1),1) = points(2:s(1),1);
                        P(conn+1:conn-1+s(1),2) = points(2:s(1),2);
                    end
                    
		    % creation of neighbors points
		    % of 4-connectivity
                    P(1,1) = x-1;
                    P(1,2) = y;
                    P(2,1) = x+1;
                    P(2,2) = y;
                    P(3,1) = x;
                    P(3,2) = y-1;
                    P(4,1) = x;
                    P(4,2) = y+1;
		    
                    % of 8-connectivity remaining
                    if conn == 8
                        
                        P(5,1) = x-1;
                        P(5,2) = y-1;
                        P(6,1) = x+1;
                        P(6,2) = y-1;
                        P(7,1) = x-1;
                        P(7,2) = y+1;
                        P(8,1) = x+1;
                        P(8,2) = y+1;
                        
                    end
                    
		    % filling the points to be remained
                    R = imfill(I,P,conn);
                    
                else
                    
		    % copy of remaining points
                    if s(1)>1
                        P(1:s(1)-1,1) = points(2:s(1),1);
                        P(1:s(1)-1,2) = points(2:s(1),2);
		    	% filling the points to be remained
                        R = imfill(I,P,conn);
                    else
                        R = I;
                    end
                    
                end
                
        else
            
	    % copy of remaining points
            if s(1)>1
                P(1:s(1)-1,1) = points(2:s(1),1);
                P(1:s(1)-1,2) = points(2:s(1),2);
                
		% filling the points to be remained
                R = imfill(I,P,conn);
            else
                R = I;
            end    
            
        end
        
    end
    
else
    
    printf('error: the number of arguments must be 2 to 3.');
    
end