%Copyright (C) 2005  Ferre-Theodore

%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

## fonction de redimensionnement d'une image
## Prends en parametre : une image avec ses trois composantes r,g,b, un facteur de dimensionnement et une méthode d'interpolation
## methode nearest : pixel calculé = valeur du plus proche pixel voisin
## methode lineaire : pixel calculé = moyenne pondérée des valeurs des 4 plus proches pixels voisins
## methode bicubic : pixel calculé = moyenne ponderée des valeur des 16 plus proches pixels voisins
## Renvoi une image avec ces trois composantes r, g ,b

function new_img = imresize2(img,scale,method)


##creation nouvelle image
new_img= (zeros(size(img,1)*scale,size(img,2)*scale,3));

##dimensions des images normales et redimenssionnees
size_img_x = size(img,1);
size_img_y = size(img,2);
size_new_img_x = size(new_img,1);
size_new_img_y = size(new_img,2);



## Methode "nearest"
if (method(3)== 'a')
##reduction
    if (scale<1)
        for i=1:size_new_img_x
            for j=1:size_new_img_y                
		## recherche de l'élément le plus proche
                l= floor(i*1.0/scale);
                p= floor(j*1.0/scale);
                new_img(i,j,:) = img(l,p,:);
            end
        end
    end 
##agrandissement
    if (scale>1)
        for i=1:size_new_img_x
            for j=1:size_new_img_y
		## recherche de l'élément le plus proche
                l= floor((i+1)*1.0/scale);
                p= floor((j+1)*1.0/scale);             
                new_img(i,j,:) = img(l,p,:);
            end
        end
    end 
end







##Methode "bicubic"
if (method(3)== 'c')
## Agrandissement et reduction  ensemble
      for i=1:size_new_img_x
            for j=1:size_new_img_y
	        fx = floor(1.0*i/scale);
       		fy = floor(1.0*j/scale);
       		dx = (1.0*i/scale) - fx;
       		dy = (1.0*j/scale) - fy;
       		som = zeros(1,3);
               	tmp2 = zeros(1,3);
                norm = 0;
		##calcul des poids pour les 16 pixels entourant le pixel recherché
       		for l=-1:2 
	       		for m=-1:2
				for u=1:4
					if ((l-dx+u-2)>0)
						p(u) = l-dx+u-2;
					else
					p(u) = 0;
					end
				end
				tmp = (1/6) * (p(4)*p(4)*p(4) - 4* p(3)* p(3)*p(3) + 6*p(2)*p(2)*p(2) -4*p(1)*p(1)*p(1));
				for u=1:4
					if ((m-dy+u-2)>0)
						p(u) = m-dy+u-2;
					else
						p(u) = 0;
					end
				end
				tmp = tmp * (1/6) * (p(4)*p(4)*p(4) - 4* p(3)* p(3)*p(3) + 6*p(2)*p(2)*p(2) -4*p(1)*p(1)*p(1));
				## Si depassement sur les bords
        			if (fx+l>size_img_x)
        			      fx=size_img_x-l;
        			end
        			if (fy+m>size_img_y)
        			      fy=size_img_y-m;
        			end
        			if (fx+l<1)
        			      fx=1-l;  
        			end
        			if (fy+m<1)
        			      fy=1-m;  
        			end
				## affectation des pixels recherchés
        			tmp2(1)= (img(fx+l,fy+m,1));
        			tmp2(2)= (img(fx+l,fy+m,2));
        			tmp2(3)= (img(fx+l,fy+m,3));
				som = som + tmp2 * tmp;
				#Calcul de la norme
				norm = norm + tmp;
			end
		end
		## Normalisation entre 0 et 255
		tmp_pix = floor(som/norm);

		new_img(i,j,1) = tmp_pix(1);
		new_img(i,j,2) = tmp_pix(2);
		new_img(i,j,3) = tmp_pix(3);

		
            end
        end
    end    
    


## Methode "bilinear"
if (method(3)== 'l')
## reduction
       if (scale<1)
         for i=1:size_new_img_x
            for j=1:size_new_img_y
                l= (i*1.0/scale);
                p= (j*1.0/scale);
		## recherche des 4  pixels les plus proches
                l1 = floor(l);
                l2 = l1+1;
                p1 = floor(p);
                p2 = p1+1;
                if l2>size_img_x
                   l2=size_img_x;
                end
                if p2>size_img_y
                   p2=size_img_y;
                end
		## Calcul du poids de chaque pixel suivant leur distance
                dl1 = abs(l1-l);
                dl2 = abs(l2-l);
                dp1 = abs(p1-p);
                dp2 = abs(p2-p);
                
                 if (dl1+ dp1 ==0) 
                     d1 = 10;
                     d2 = 1;
                     d3 = 1;
                     d4 = 1;
                else
                    d1 = 1/sqrt(dl1*dl1 + dp1*dp1);
                    d2 = 1/sqrt(dl1*dl1 + dp2*dp2);
                    d3 = 1/sqrt(dl2*dl2 + dp1*dp1);
                    d4 = 1/sqrt(dl2*dl2 + dp2*dp2);
                end
                ## Calcul du pixel recherché
                tmp = d1 * img(l1,p1,:) + d2 * img(l1,p2,:) + d3 * img(l2,p1,:) + d4 * img(l2,p2,:);
                new_img(i,j,:) = floor(tmp  / (d1+d2+d3+d4));
            end
        end
        
    end 
## Agrandissement
    if (scale>1)
       for i=1:size_new_img_x
            for j=1:size_new_img_y
                l= i*1.0/scale;
                p= j*1.0/scale;
		## recherche des 4  pixels les plus proches
                l1 = floor(l);
                l2 = l1+1;
                p1 = floor(p);
                p2 = p1+1;
                if l1<1
                   l1=1;
                end
                if p1<1
                   p1=1;
                end
                
                if l2>size_img_x
                   l2=size_img_x;
                end
                if p2>size_img_y
                   p2=size_img_y;
                end
		## Calcul du poids de chaque pixel suivant leur distance
                dl1 = abs(l1-l);
                dl2 = abs(l2-l);
                dp1 = abs(p1-p);
                dp2 = abs(p2-p);
                
	
                 if (dl1+ dp1 ==0) 
                     d1 = 10;
                     d2 = 1;
                     d3 = 1;
                     d4 = 1;
                else
                    d1 = 1/sqrt(dl1*dl1 + dp1*dp1);
                    d2 = 1/sqrt(dl1*dl1 + dp2*dp2);
                    d3 = 1/sqrt(dl2*dl2 + dp1*dp1);
                    d4 = 1/sqrt(dl2*dl2 + dp2*dp2);
                end
                
		## Calcul du pixel recherché
		tmp = d1 * img(l1,p1,:) + d2 * img(l1,p2,:) + d3 * img(l2,p1,:) + d4 * img(l2,p2,:);
                new_img(i,j,:) = floor(tmp  / (d1+d2+d3+d4));
            end
        end
    end 
end
