%Copyright (C) 2005  Grucker-Hardy

%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

%#    RADON Compute Radon transform.
%#    The RADON function computes the Radon transform, which is the
%#    projection of the image intensity along a radial line
%#    oriented at a specific angle.
%# 
%#    R = RADON(I,THETA,N) returns the Radon transform of the
%#    intensity image I for the angle THETA degrees. If THETA is a
%#    scalar, the result R is a column vector containing the Radon
%#    transform for THETA degrees. If THETA is a vector, then R is
%#    a matrix in which each column is the Radon transform for one
%#    of the angles in THETA.
%# 
%#    R = RADON(I,THETA,N) returns a Radon transform with the
%#    projection computed at N points. R has N rows. 
%# 
%#    [R,Xp] = RADON(...) returns a vector Xp containing the radial
%#    coordinates corresponding to each row of R.
    
function [R,Xp] = radon(I,theta,Nb)


PI = 3.14159;

theta = theta / 180;
theta = theta * PI;

s = size(I);
s2 = size(theta);
s2(2);

N = Nb;
coeff = (2*ceil(norm(size(I)-floor((size(I)-1)/2)-1))+3)/Nb;

x0 = floor(s(1)/2);
y0 = floor(s(2)/2);

R = zeros(N,s2(2));


for i=1:s2(2)
	
	for m=-floor((N-1)/2):floor((N-1)/2)
	
		x1 = coeff*m * cos(theta(i));
		y1 = - coeff*m * sin(theta(i));

		for n=-floor((N-1)/2):floor((N-1)/2)
		
			x = x1 + coeff*n*sin(theta(i));
			y = y1 + coeff*n*cos(theta(i));
		
			x = floor(x0+x);
			y = floor(y0+y);
		
			if ((x<s(1))&&(y<(s(2))) && (x>0)&&(y>0))
				R(N-(m+floor((N+1)/2))+1,i) = R(N-(m+floor((N+1)/2))+1,i) + I(x,y);
			end

		end	
	end	
end

R = floor( ( R/ max(max(R)) ) *255);


